#!/bin/sh
#
# iiia MUST have awk, sed, grep, tr, egrep, cat, xargs and echo to run. 
# If you are on a system that does not have one of these (e.g., WINDOWS),
#  you have bigger problems than getting iiia to work.

#
# Determine automatically where our home is.
# resolve links - $0 may be a softlink
PRG="$0"
while [ -h "$PRG" ] ; do
  ls=`ls -ld "$PRG"`
  link=`expr "$ls" : '.*-> \(.*\)$'`
  if expr "$link" : '.*/.*' > /dev/null; then
    PRG="$link"
  else
    PRG=`dirname "$PRG"`/"$link"
  fi
done
PRGDIR=`dirname "$PRG"`

# Set environment
if [ -f $PRGDIR/setIscEnv.sh ] ; then
	. $PRGDIR/setIscEnv.sh
else
	echo "There is no environment setup file: setIscEnv.sh"
	echo "Please re-run configureWily in installation directory!"
	exit
fi

WORKING_DIR=/tmp
PROBEBUILDER=$ISC_HOME/lib/ProbeBuilder.jar
BCA=$ISC_HOME/lib/ByteCodeAnalyzer.jar
IIIA=$ISC_HOME/lib/iiia.jar
FINALPBD=./iiia.pbd
USAGE="usage: iiia.sh (-lsof <pid> | -archives <jar|ear|war>*)"


ANALYZE="$ISC_JAVA_HOME/bin/java -Xms128M -Xmx256M -cp $BCA:$PROBEBUILDER com.wily.tools.bytecodeanalyzer.HierarchyByteCodeAnalyzer"
EGREP=egrep
AWK=awk
SED=sed
TR=tr
LSOF="/usr/sbin/lsof -p"

####### EDIT BELOW HERE AT YOUR OWN RISK #######

#check command line for options
if [ -z $1 ]; 
then echo -e \\n"You must specify how iiia is to look for archives to analyze."; 
     echo -e \\n$USAGE;
     echo;
     exit 0;
fi;

if [ $1 = "-lsof" ];
then mode=lsof;
elif [ $1 = "-archives" ];
then mode=archives;
else echo -e \\n"You must specify how iiia is to look for archives to analyze."; 
     echo -e \\n$USAGE;
     echo;
     exit 0;
fi;

#clear out archive list
echo > $WORKING_DIR/introscope.iiia.archive-list

#create archive list from lsof
if [ $mode = lsof ];
then 
	if [ $2 -gt 0 ];
	then
		echo -n "Using lsof to locate jar, war and ear files ...";
		$LSOF $2 | $EGREP \(jar\|war\|ear\) | $AWK '{print $9}' > $WORKING_DIR/introscope.iiia.archive-list
		echo  done.;
	else echo -e \\n"Please specify a process id to use with lsof."; 
 	    	echo "usage: iiia.sh (-lsof <pid> | -archives <jar|ear|war>+)";
     		echo;
     		exit 0;
	fi;
fi;

#create archive list file from command line
if [ $mode = archives ];
then echo Checking archives ...  ${@:2:$#};
	for x in ${@:2:$#}; do 
		if [ -f $x ]; then
		   echo $x >> $WORKING_DIR/introscope.iiia.archive-list;
		else 
		   echo "$x is not a file.";
		fi;
	done;
    echo "... done."
fi;

#clear out bca output file
echo > $WORKING_DIR/introscope.iiia.bca-dirty

#create bca output
cat $WORKING_DIR/introscope.iiia.archive-list | xargs -n1 $ANALYZE  >> $WORKING_DIR/introscope.iiia.bca-dirty

#reformat bca output
cat $WORKING_DIR/introscope.iiia.bca-dirty | $SED -e 's/Class //g' | $SED -e 's/Interface//g' | $TR '<' ' ' | $TR '>' ' ' | $SED -e 's/ //g' > $WORKING_DIR/introscope.iiia.bca-clean

#create parent list
grep IdentifyInheritedAs $ISC_HOME/*.pbd | grep -v \# | $AWK {'print $2 " " $3'} > $WORKING_DIR/introscope.iiia.inherited-list

#run iiia against parent list + prettied up bca output
$ISC_JAVA_HOME/bin/java -jar $IIIA $WORKING_DIR/introscope.iiia.inherited-list $WORKING_DIR/introscope.iiia.bca-clean > $FINALPBD

echo "Created iiia pbd in $FINALPBD."

echo "iiia has finished".
